'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compilerCore = require('@vue/compiler-core');

const generateUtil = {
    isRoot(node) {
        return node && node.type === 0;
    },
    isNode(node) {
        return node && node.type === 1;
    },
    isText(node) {
        return node && node.type === 2;
    },
    isExpr(node) {
        return !!node && node.type === 5;
    },
    isSimpleExpr(node) {
        return !!node && node.type === 4;
    },
    isAttr(node) {
        return node && node.type === 6;
    },
    isDirective(node) {
        return node && node.type === 7;
    },
};
const defaultGenFns = {
    genNodeName(name, { push }) {
        push(name);
    },
    genProps(props, context) {
        if (props && props.length > 0) {
            const { isAttr, isDirective, genAttr, genDirective } = context;
            props.forEach((p) => {
                if (isAttr(p)) {
                    genAttr(p, context);
                }
                else if (isDirective(p)) {
                    genDirective(p, context);
                }
            });
        }
    },
    genText(text, { push }) {
        push(text);
    },
    genTextExpr(node, { push }) {
        push('{{' + node.content + '}}');
    },
    genStringValue(text, { push }) {
        push(JSON.stringify(text));
    },
    genPropsExpr(node, context) {
        const { propsType, push, genStringValue } = context;
        if (propsType === 'value') {
            genStringValue(node.content, context);
        }
        else {
            push(node.content);
        }
    },
    genAttr(attr, context) {
        const { push, isText, genStringValue } = context;
        push(' ' + attr.name);
        if (attr.value && isText(attr.value)) {
            push('=');
            genStringValue(attr.value.content, context);
        }
    },
    genPropsEvent(directive, context) {
        const { push, isSimpleExpr, genPropsExpr } = context;
        if (isSimpleExpr(directive.arg)) {
            push(' @');
            genPropsExpr(directive.arg, Object.assign(Object.assign({}, context), { propsType: 'key' }));
        }
    },
    genPropsBind(directive, context) {
        const { push, isSimpleExpr, genPropsExpr } = context;
        if (isSimpleExpr(directive.arg)) {
            if (directive.arg) {
                push(' :');
                genPropsExpr(directive.arg, Object.assign(Object.assign({}, context), { propsType: 'key' }));
            }
            else {
                push(' v-bind');
            }
        }
    },
    genPropsCustomDirective(directive, { push }) {
        push(' v-' + directive.name);
    },
    genDirective(directive, context) {
        const { push, isSimpleExpr, genPropsExpr, genPropsBind, genPropsEvent, genPropsCustomDirective, } = context;
        if (directive.name === 'on') {
            genPropsEvent(directive, context);
        }
        else if (directive.name === 'bind') {
            genPropsBind(directive, context);
        }
        else {
            genPropsCustomDirective(directive, context);
        }
        if (isSimpleExpr(directive.exp)) {
            push('=');
            genPropsExpr(directive.exp, Object.assign(Object.assign({}, context), { propsType: 'value' }));
        }
    },
    genNode(node, context) {
        const { push, genNodeName, genProps } = context;
        push('<');
        genNodeName(node.tag, context, node);
        genProps(node.props, context, node);
        if (node.isSelfClosing) {
            push(' />');
        }
        else {
            push('>');
        }
        return function () {
            if (!node.isSelfClosing) {
                push('</');
                genNodeName(node.tag, context, node);
                push('>');
            }
        };
    },
};
const defaultNodeGenerate = function (node, context) {
    const { isRoot, isNode, isText, isExpr, isSimpleExpr, genText, genNode, genTextExpr, } = context;
    if (isRoot(node))
        return;
    if (isText(node)) {
        genText(node.content, context, node);
    }
    else if (isNode(node)) {
        return genNode(node, context);
    }
    else if (isExpr(node)) {
        if (isSimpleExpr(node.content)) {
            genTextExpr(node.content, context);
        }
    }
};

function compiler(template, options = {}) {
    const ast = typeof template === 'string' ? compilerCore.baseParse(template, options) : template;
    compilerCore.transform(ast, Object.assign({}, options, {
        //   prefixIdentifiers,
        nodeTransforms: [
            ...(options.nodeTransforms || []), // user transforms
        ],
    }));
    const context = createGenerateContext(options);
    generate(ast, context);
    return {
        code: context.code,
    };
}
function generate(node, context) {
    const { nodeGenerate } = context;
    const onExit = nodeGenerate && nodeGenerate(node, context);
    if (node.type === 1 || node.type === 0) {
        for (let c = 0; c < node.children.length; c++) {
            generate(node.children[c], context);
        }
    }
    onExit && onExit(node, context);
}
function createGenerateContext({ scopeId = null, nodeGenerate = defaultNodeGenerate, genFn, } = {}) {
    const context = Object.assign(Object.assign({ code: '', scopeId,
        nodeGenerate,
        push(code) {
            context.code += code;
        } }, generateUtil), Object.assign({}, defaultGenFns, genFn));
    return context;
}

exports.compiler = compiler;
